@@ -1,5 +1,5 @@
 /* GDA library
- * Copyright (C) 1998 - 2006 The GNOME Foundation.
+ * Copyright (C) 1998 - 2008 The GNOME Foundation.
  *
  * AUTHORS:
  *      Michael Lausch <michael@lausch.at>
@@ -26,29 +26,30 @@
 #ifndef __LIBGDA_H__
 #define __LIBGDA_H__
 
+#include <libgda/gda-attributes-manager.h>
 #include <libgda/gda-easy.h>
-#include <libgda/gda-column-index.h>
 #include <libgda/gda-column.h>
 #include <libgda/gda-config.h>
 #include <libgda/gda-connection-event.h>
 #include <libgda/gda-connection.h>
 #include <libgda/gda-connection-private.h>
+#include <libgda/gda-data-comparator.h>
 #include <libgda/gda-data-model-array.h>
 #include <libgda/gda-data-model-bdb.h>
-#include <libgda/gda-data-model-index.h>
-#include <libgda/gda-data-model-query.h>
 #include <libgda/gda-data-model.h>
 #include <libgda/gda-data-model-iter.h>
 #include <libgda/gda-data-model-import.h>
 #include <libgda/gda-data-model-dir.h>
 #include <libgda/gda-data-access-wrapper.h>
 #include <libgda/gda-data-proxy.h>
+#include <libgda/gda-data-select.h>
+#include <libgda/gda-lockable.h>
 #include <libgda/gda-log.h>
 #include <libgda/gda-quark-list.h>
 #include <libgda/gda-row.h>
 #include <libgda/gda-server-operation.h>
 #include <libgda/gda-server-provider.h>
-#include <libgda/gda-threader.h>
+#include <libgda/gda-xa-transaction.h>
 #include <libgda/gda-transaction-status.h>
 #include <libgda/gda-transaction-status-private.h>
 #include <libgda/gda-util.h>
@@ -72,13 +73,12 @@
 
 #include <libgda/gda-meta-store.h>
 
+#include <libgda/gda-mutex.h>
+
 G_BEGIN_DECLS
 
-void     gda_init             (const gchar *app_id, const gchar *version, gint nargs, gchar *args[]);
-
-typedef  void (* GdaInitFunc) (gpointer user_data);
-void     gda_main_run         (GdaInitFunc init_func, gpointer user_data);
-void     gda_main_quit        (void);
+void gda_init (void);
+gchar *gda_get_application_exec_path (const gchar *app_name);
 
 G_END_DECLS
 
