@@ -204,7 +204,7 @@ gda_vconnection_hub_add (GdaVconnectionHub *hub,
 	/* check for constraints */
 	hc = get_hub_cnc_by_ns (hub, ns);
 	if (hc && (hc->cnc != cnc)) {
-		g_set_error (error, 0, 0,
+		g_set_error (error, 0, 0, "%s", 
 			     _("Namespace must be specified"));
 		return FALSE;
 	}
@@ -213,7 +213,7 @@ gda_vconnection_hub_add (GdaVconnectionHub *hub,
 		return TRUE;
 
 	if (!gda_connection_is_opened (cnc)) {
-		g_set_error (error, 0, 0,
+		g_set_error (error, 0, 0, "%s", 
 			     _("Connection is closed"));
 		return FALSE;
 	}
@@ -254,7 +254,7 @@ gda_vconnection_hub_remove (GdaVconnectionHub *hub, GdaConnection *cnc, GError *
 	hc = get_hub_cnc_by_cnc (hub, cnc);
 
 	if (!hc) {
-		g_set_error (error, 0, 0,
+		g_set_error (error, 0, 0, "%s", 
 			     _("Connection was not represented in hub"));
 		return FALSE;
 	}
@@ -313,7 +313,7 @@ gda_vconnection_hub_get_connection (GdaVconnectionHub *hub, const gchar *ns)
 /**
  * gda_vconnection_hub_foreach
  * @hub: a #GdaVconnectionHub connection
- * @func: a #GdaVConnectionDataModelFunc function pointer
+ * @func: a #GdaVconnectionDataModelFunc function pointer
  * @data: data to pass to @cunc calls
  *
  * Call @func for each #GdaConnection represented in @hub.
@@ -422,7 +422,7 @@ attach_hub_connection (GdaVconnectionHub *hub, HubConnection *hc, GError **error
 	GdaMetaStore *store;
 	GdaMetaContext context;
 	
-	g_object_get (G_OBJECT (hc->cnc), "meta-store", &store, NULL);
+	store = gda_connection_get_meta_store (hc->cnc);
 	g_assert (store);
 
 	/* make sure the meta store is up to date */
@@ -451,10 +451,21 @@ attach_hub_connection (GdaVconnectionHub *hub, HubConnection *hc, GError **error
 	model = gda_connection_get_meta_store_data (hc->cnc, GDA_CONNECTION_META_TABLES, error, 0);
 	if (!model)
 		return FALSE;
+
 	nrows = gda_data_model_get_n_rows (model);
 	for (i = 0; i < nrows; i++) {
 		const GValue *cv = gda_data_model_get_value_at (model, 0, i, error);
-		if (!cv || !table_add (hc, cv, error)) {
+		const GValue *cv1 = gda_data_model_get_value_at (model, 2, i, error);
+		if (!cv || !cv1) {
+			g_object_unref (model);
+			return FALSE;
+		}
+
+		/* ignore tables which require a complete name <schema>.<name> */
+		if (!gda_value_differ (cv, cv1))
+			continue;
+
+		if (!table_add (hc, cv, error)) {
 			g_object_unref (model);
 			return FALSE;
 		}
